# Generated by Django 3.2.5 on 2021-10-12 16:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pageblocks', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='pageblock',
            name='content',
        ),
        migrations.AddField(
            model_name='pageblock',
            name='data',
            field=models.JSONField(default=dict),
        ),
    ]
