{% extends "admin/change_form.html" %}
{% load static %}

{% block extrahead %}{{ block.super }}
<script src="https://cdn.jsdelivr.net/npm/vue@2.6.14" defer></script>
<script type="text/javascript" src="{% static 'admin/pageblocks/js/change_form.js' %}" defer></script>
{% endblock %}

{% block extrastyle %}{{ block.super }}
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.4/css/all.min.css" integrity="sha512-1ycn6IcaQQ40/MKBW2W4Rhis/DbILU74C1vSrLJxCq57o941Ym01SwNsOMqvEBFlcgUa6xLiPY/NS5R+E6ztJQ==" crossorigin="anonymous" referrerpolicy="no-referrer" />
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/flag-icon-css/3.5.0/css/flag-icon.min.css" integrity="sha512-Cv93isQdFwaKBV+Z4X8kaVBYWHST58Xb/jVOcV9aRsGSArZsgAnFIhMpDoMDcFNoUtday1hdjn0nGp3+KZyyFw==" crossorigin="anonymous" referrerpolicy="no-referrer" />
<link rel="stylesheet" type="text/css" href="{% static "admin/pageblocks/css/change_form.css" %}">
{% endblock %}