# Generated by Django 3.2.5 on 2021-11-07 17:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pageblocks', '0002_auto_20211012_1854'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='pageblock',
            options={'ordering': ['index']},
        ),
        migrations.AddField(
            model_name='pageblock',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='pageblocks.pageblock'),
        ),
    ]
