# Generated by Django 3.2.5 on 2021-10-12 16:50

from django.db import migrations, models
import django.db.models.deletion
import pageblocks.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('slug', models.SlugField(unique=True)),
                ('title', pageblocks.models.MultiLanguageField(default=dict)),
            ],
        ),
        migrations.CreateModel(
            name='PageBlock',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('language', models.CharField(db_index=True, max_length=2)),
                ('index', models.IntegerField(default=-1)),
                ('type', models.TextField()),
                ('content', models.TextField()),
                ('page', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='blocks', to='pageblocks.page')),
            ],
        ),
    ]
